//-----------------------------------------------------------------------------
//
// Program Description:
// --------------------
//
// This code takes and averages 10 digital measurements from input
// P1.4 (PING Ultrasonic Distance Sensor). It then uses that information
// to calculate and print the distance (in inches), to a target object,
// to an 8-digit LCD screen.
// If the measured distance is:
//   - more than 97 inches, then external lights will turn OFF.
//   - between 96 and 31 inches, then external lights will blink.
//   - less than 30 inches, then external lights will turn ON.
//
// The system is clocked by the internal 20MHz oscillator.  Timer 0 triggers
// on its overflow every 5us, which, in turn, generates a 5us timer.
//
// When the averaging of 10 measurements is complete the value is printed
// before starting another average cycle.
//
//
// F93x Resources:
// ---------------
// Timer0: overflow initiates 5us timer tick.
//
// How To Test:
// ------------
// 1) Ensure that jumpers are placed on the following:
//       J12:  VDD/VIO
//       J11:  PWR/WALL_PWR
//       J17:  VDD_PIN/PWR
// 2) Connect the hardware as shown in the associated schematics/connectivity
//	  diagram.
// 3) Connect a USB cable from the development board P3 to a PC.
// 4) Connect the PING Ultrasonic Distance Sensor's SIG to P1.4.
// 5) Connect the USB Debug Adapter to J9.
// 6) Turn the POWER switch (SW5) to the "ON" position.
// 7) Compile and download code to a 'F93x device on a C8051F93x-TB development
//    board by selecting Run -> Debug from the menus, clicking the Debug button
//    in the quick menu, or pressing F11.
// 8) Run the code by selecting Run -> Resume from the menus, clicking the
//    Resume button in the quick menu, or pressing F8.
// 9) The CP2400-GQ LCD dev board will display the measured data.
// 10) Move a target such that it is:
//		- 97 inches (or more) from the sensor; the lights should be OFF.
//		- between 96 and 31 inches; the lights should blink.
//		- less than 30 inches; the lights should be ON.
// 11) The PING's Ultrasonic Distance Sensor range is advertised as: 2 cm to 3 m (0.8 to 118 inches).
//
// Target:         C8051F92x-C8051F93x

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <compiler_defs.h>
#include <SI_C8051F930_Register_Enums.h>	// SFR declarations
#include "InitDevice.h"
#include <stdio.h>
#include "..\LCD_LIB\lcd_lib.h"

//-----------------------------------------------------------------------------
// Pin Declarations
//-----------------------------------------------------------------------------
SBIT (PingSig, SFR_P1, 4);		// P1.4. This signal will both drive and monitor the PING's signal.
SBIT (LightsOn, SFR_P2, 1);		// P2.1. This port will turn ON/OFF the external lights.

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------
char i = 0;
char xdata display_string[9];
char DisplayResults;
char LightOnFlag = 0;			// Declare variable and initialize to 0.
char LightOffFlag = 1;			// Declare variable and initialize to 1.
char LightBlinkFlag = 0;		// Declare variable and initialize to 0.

U8 AirTemperature = 20.0;		// The ambient air temperature (in degrees C).

U16 AverageMeasurements = 10;	// Declare variable and initialize to 10.
U16 LightBlinkCounter = 1;		// Declare variable and initialize.

U32 EchoTimerMeasured = 0;		// Declare variable and initialize to 0.

float AverageAccumulator = 0;	// Declare variable and initialize to 0.
float MeasuredDistance = 0;		// Declare variable and initialize to 0.
float MeasuredDistanceFeet = 0;	// Declare variable and initialize to 0.
float MeasuredDistanceInches = 0;// Declare variable and initialize to 0.

extern U16 MICROSECOND_TIMER1;	// Externally declared variable.
extern U32 EchoTimer;			// Declare variable.

//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
int main (void)
{
	//-------------------------------------------------
	//Enter default mode (configure the microprocessor)
	//-------------------------------------------------
	enter_DefaultMode_from_RESET();

	//----------------------------------
	// Indicate TX0 ready.
	//----------------------------------
	SCON0_TI = 1;

	//----------------------------------
	// Basic CP240x Initialization
	//----------------------------------
	CP240x_Reset();
	CP240x_RTC_Init();

	//----------------------------------
	// LCD Display Application Code
	//----------------------------------
	LCD_Init();
	LCD_Clear();

	//----------------------------------
	// Main Application Loop
	//----------------------------------
	PingSig = 0;					// Set Ping Signal to a low level.

	while (1)
	{
		P1MDOUT = (P1MDOUT | 0x10);	// Change P1.4 (the port connected to the PING's signal) to an output.
		EchoTimer = 0;
		PingSig = 1;				// Set Ping Signal to a high level to initiate Sonar TX.
		_nop_ ();					// ~3.3 us delay. Hold signal high for 2us min (5us typical) per the PING's datasheet.
		MICROSECOND_TIMER1 = 0;		// Reset
		PingSig = 0;				// Set Ping Signal to a low level to complete the initiate Sonar TX routine.

		while(MICROSECOND_TIMER1 < 170)	// Hold signal low for 750us. 170 actually equates to 850us (170 * 5us = 850us),
										// where 5us is the MICROSECOND_TIMER1 timer duration. 850us was proven to be
										// necessary during testing.
		{
			// do nothing
		}

		P1MDOUT = (P1MDOUT & 0xEF);	// Change P1.4 to an input.
		P1 = 0xF7;					// Change P1.4 latch setting to high (high impedance).
		MICROSECOND_TIMER1 = 0;		// Reset
		EchoTimerMeasured = 0;		// Reset
		EchoTimer = 0;				// Reset

		while(PingSig == 1)
		{
			// do nothing
		}

		EchoTimerMeasured = EchoTimer;

		// Measure distance in meters.
		// Note: This is the distance to and from the target (i.e., the sound travels to the target, reflects off it,
		// then travels back to the sensor).
		MeasuredDistance = (331.5 + (0.606 * (float)AirTemperature)) * (float) EchoTimerMeasured * 5.0/1000000;

		MeasuredDistance = (MeasuredDistance / 2.0);			// Measured distance in meters (distance to target only).
		MeasuredDistanceFeet = (MeasuredDistance * 3.28084);	// Measured distance in feet.
		MeasuredDistanceInches = (MeasuredDistanceFeet * 12.0);	// Measured distance in inches.

		AverageAccumulator += MeasuredDistanceInches;			// Used for averaging measurements.
		AverageMeasurements--;									// Decrement the measurement counter.

		// Averaging measurements.
		if(AverageMeasurements == 0)
		{
			MeasuredDistanceInches = (AverageAccumulator/10.0);
			AverageAccumulator = 0;
			AverageMeasurements = 10;
			DisplayResults = 1;
		}

		// Delay for 5ms (4000*5us=20ms). This allows enough time for the maximum echo return pulse (18.5ms)
		// and the delay before next measurement (200us), per the PING's datasheet.
		while(MICROSECOND_TIMER1 < 4000)
		{
			// do nothing
		}

		//-----------------------------------------------------------------------------
		// Display Value Routine
		// ----------------------------------------------------------------------------

		if(DisplayResults == 1)
		{
			// Serial port.
			printf("\nDistance: %3.3f inches          ", (float) MeasuredDistanceInches);	// Note: the format "%3.3f" --> "3" = width of 3 positions; ".3" = 3 digits after decimal point;
			printf("\033[0;0H");															// Set cursor to home position on screen.
			MeasuredDistanceInches = MeasuredDistanceInches*10.0;
			sprintf(display_string, "%04d in ", (unsigned int) MeasuredDistanceInches);		// Note: the format "%04d" --> "0" = Left-pads the number with zeroes (0) instead of spaces.
																							// "4" = width of 4 positions; "d" = Signed decimal integer.

			LCD_OutString(display_string);													// Send data to LCD.
			DisplayResults = 0;																// Reset flag.

			// If the measured distance is less than 30 inches, then turn ON the lights.
			if(MeasuredDistanceInches <= 300)
			{
				LightsOn = 1;
				LightOnFlag = 1;	//	Set flag signifying that the lights are ON.
				LightOffFlag = 0;	//	Set flag signifying that the lights are not OFF.
				LightBlinkFlag = 0;	//	Set flag signifying that the lights are not blinking.
			}

			// If the measured distance is less than or equal to 96 and greater than or equal to 31 inches, then blink the lights.
			else if((MeasuredDistanceInches <= 960) && (MeasuredDistanceInches >= 310))
			{
				if((LightsOn == 1) && (LightBlinkCounter == 0))
				{
					LightsOn = 0;
					LightBlinkCounter = 1;
					LightOnFlag = 0;	//	Set flag signifying that the lights are not ON.
					LightOffFlag = 0;	//	Set flag signifying that the lights are not OFF.
					LightBlinkFlag = 1;	//	Set flag signifying that the lights are blinking.
				}

				else if((LightsOn == 1) && (LightBlinkCounter > 0))
				{
					LightBlinkCounter--;
				}

				else if((LightsOn == 0) && (LightBlinkCounter == 0))
				{
					LightsOn = 1;
					LightBlinkCounter = 1;
				}

				else if((LightsOn == 0) && (LightBlinkCounter > 0))
				{
					LightBlinkCounter--;
				}
			}

			// If the measured distance is less than 97 inches and greater than 96 inches,
			// and the lights are currently off, then leave the lights off.
			else if((MeasuredDistanceInches < 970) && (MeasuredDistanceInches > 960) && (LightOffFlag == 1))
			{
				LightsOn = 0;
				LightOnFlag = 0;	//	Set flag signifying that the lights are not ON.
				LightOffFlag = 1;	//	Set flag signifying that the lights are OFF.
				LightBlinkFlag = 0;	//	Set flag signifying that the lights are not blinking.
			}

			// If the measured distance is less than 97 inches and greater than 96 inches,
			// and the lights are currently blinking, then leave the lights blinking.
			else if((MeasuredDistanceInches < 970) && (MeasuredDistanceInches > 960) && (LightBlinkFlag == 1))
			{
				LightOnFlag = 0;	//	Set flag signifying that the lights are not ON.
				LightOffFlag = 0;	//	Set flag signifying that the lights are not OFF.
				LightBlinkFlag = 1;	//	Set flag signifying that the lights are blinking.

				if((LightsOn == 1) && (LightBlinkCounter == 0))
				{
					LightsOn = 0;
					LightBlinkCounter = 1;
					LightOnFlag = 0;	//	Set flag signifying that the lights are not ON.
					LightOffFlag = 0;	//	Set flag signifying that the lights are not OFF.
					LightBlinkFlag = 1;	//	Set flag signifying that the lights are blinking.
				}

				else if((LightsOn == 1) && (LightBlinkCounter > 0))
				{
					LightBlinkCounter--;
				}

				else if((LightsOn == 0) && (LightBlinkCounter == 0))
				{
					LightsOn = 1;
					LightBlinkCounter = 1;
				}

				else if((LightsOn == 0) && (LightBlinkCounter > 0))
				{
					LightBlinkCounter--;
				}
			}

			// If the measured distance is less than 31 inches and greater than 30 inches,
			// and the lights are either currently on or off (off would occur at startup), then leave/turn the lights on.
			else if((MeasuredDistanceInches < 310) && (MeasuredDistanceInches > 300) && ((LightOnFlag == 1) || (LightOffFlag == 1)))
			{
				LightsOn = 1;
				LightOnFlag = 1;	//	Set flag signifying that the lights are ON.
				LightOffFlag = 0;	//	Set flag signifying that the lights are not OFF.
				LightBlinkFlag = 0;	//	Set flag signifying that the lights are not blinking.
			}

			// If the measured distance is less than 31 inches and greater than 30 inches,
			// and the lights are currently blinking, then leave the lights blinking.
			else if((MeasuredDistanceInches < 310) && (MeasuredDistanceInches > 300) && (LightBlinkFlag == 1))
			{
				LightOnFlag = 0;	//	Set flag signifying that the lights are not ON.
				LightOffFlag = 0;	//	Set flag signifying that the lights are not OFF.
				LightBlinkFlag = 1;	//	Set flag signifying that the lights are blinking.

				if((LightsOn == 1) && (LightBlinkCounter == 0))
				{
					LightsOn = 0;
					LightBlinkCounter = 1;
					LightOnFlag = 0;	//	Set flag signifying that the lights are not ON.
					LightOffFlag = 0;	//	Set flag signifying that the lights are not OFF.
					LightBlinkFlag = 1;	//	Set flag signifying that the lights are blinking.
				}

				else if((LightsOn == 1) && (LightBlinkCounter > 0))
				{
					LightBlinkCounter--;
				}

				else if((LightsOn == 0) && (LightBlinkCounter == 0))
				{
					LightsOn = 1;
					LightBlinkCounter = 1;
				}

				else if((LightsOn == 0) && (LightBlinkCounter > 0))
				{
					LightBlinkCounter--;
				}
			}

			// If the measured distance is more than or equal to 970 inches, then turn OFF the lights.
			else if(MeasuredDistanceInches >= 970)
			{
				LightsOn = 0;
				LightOnFlag = 0;	//	Set flag signifying that the lights are not ON.
				LightOffFlag = 1;	//	Set flag signifying that the lights are OFF.
				LightBlinkFlag = 0;	//	Set flag signifying that the lights are not blinking.
			}
		}
	}	// while (1). Do forever.
}
